package de.is24.realEstates.domain.energycertificate.validation;

import de.is24.realEstates.domain.energyCertificate.EnergyCertificateCreationDate;
import de.is24.realEstates.domain.energyCertificate.EnergyEfficiencyClass;
import de.is24.realEstates.domain.energycertificate.RealEstateWithEnergyCertificate;
import de.is24.realEstates.domain.energycertificate.validation.rules.EmptyValidationRule;
import de.is24.realEstates.domain.energycertificate.validation.rules.EnergyCertificateValidationRule;
import de.is24.realEstates.domain.energycertificate.validation.rules.NumberValidationRule;
import de.is24.realEstates.domain.living.BuildingEnergyRatingType;
import de.is24.realEstates.domain.living.FiringType;
import de.is24.realEstates.domain.living.HeatingType;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Set;
import static de.is24.common.validation.ValidationConstants.NUMBER_IS_OUT_OF_RANGE;
import static de.is24.realEstates.domain.energycertificate.validation.EnergyCertificateValidationConstants.MAX_THERMAL_CHARACTERISTIC;
import static de.is24.realEstates.domain.energycertificate.validation.EnergyCertificateValidationConstants.NUMBER_MAX_VALUE;
import static de.is24.realEstates.domain.energycertificate.validation.EnergyCertificateValidationConstants.NUMBER_MIN_VALUE;
import static de.is24.util.ConstraintUtils.notBlank;


public enum EnergyCertificateField implements EnergyCertificateValidationErrorCodes {
  //Energieverbrauch fur Warmwasser enthalten
  //Nested class value need to be specified by developer when call validation
  EV_THERMAL_CHARACTERISTIC_WARM_WATER_INC(new EmptyValidationRule(), "energyConsumptionContainsWarmWater",
    EV_WARM_WATER_FIELD_NOT_VALID_FOR_THIS_CERTIFICATE) {
    @Override
    public void set(RealEstateWithEnergyCertificate realEstateWithEnergyCertificate, Object value) {
      Boolean valueToPass = (Boolean) value;
      if (value == null) {
        valueToPass = Boolean.FALSE;
      }
      realEstateWithEnergyCertificate.wrapAsEnergyCertificateRelatedProperties()
      .setEnergyConsumptionContainsWarmWater(valueToPass);
    }

    @Override
    public Object get(RealEstateWithEnergyCertificate realEstateWithEnergyCertificate) {
      return realEstateWithEnergyCertificate.wrapAsEnergyCertificateRelatedProperties()
        .getEnergyConsumptionContainsWarmWater();
    }
  },

  //Endenergiebedarf, Energieverbrauchskennwert, Endenergieverbrauch
  //Nested class value need to be specified by developer when call validation
  EV_THERMAL_CHARACTERISTIC(new NumberValidationRule(
      NUMBER_IS_OUT_OF_RANGE,
      NUMBER_MIN_VALUE,
      MAX_THERMAL_CHARACTERISTIC), "thermalCharacteristic", EV_THERMAL_CHARACTERISTIC_NOT_VALID_FOR_THIS_CERTIFICATE) {
    @Override
    public void set(RealEstateWithEnergyCertificate realEstateWithEnergyCertificate, Object value) {
      realEstateWithEnergyCertificate.wrapAsEnergyCertificateRelatedProperties()
      .setThermalCharacteristic((BigDecimal) value);
    }

    @Override
    public Object get(RealEstateWithEnergyCertificate realEstateWithEnergyCertificate) {
      return realEstateWithEnergyCertificate.wrapAsEnergyCertificateRelatedProperties().getThermalCharacteristic();
    }
  },

  //Energieauswiesdatum
  EV_ENERGY_CERTIFICATE_CREATION_DATE(new EmptyValidationRule(), "energyCertificateInformation",
    "energyCertificateCreationDate",
    EnergyCertificateValidationErrorCodes.EV_CERT_CREATION_DATE_NOT_VALID_FOR_THIS_CERTIFICATE) {
    @Override
    public void set(RealEstateWithEnergyCertificate realEstateWithEnergyCertificate, Object value) {
      realEstateWithEnergyCertificate.wrapAsEnergyCertificateRelatedProperties()
      .setEnergyCertificateCreationDate((EnergyCertificateCreationDate) value);
    }

    @Override
    public Object get(RealEstateWithEnergyCertificate realEstateWithEnergyCertificate) {
      return realEstateWithEnergyCertificate.wrapAsEnergyCertificateRelatedProperties()
        .getEnergyCertificateCreationDate();
    }
  },

  //Energieausweistype
  EV_ENERGY_CERTIFICATE_TYPE(new EmptyValidationRule(), "buildingEnergyRatingType",
    EV_CERTIFICATE_TYPE_NOT_VALID_FOR_THIS_CERTIFICATE) {
    @Override
    public void set(RealEstateWithEnergyCertificate realEstateWithEnergyCertificate, Object value) {
      realEstateWithEnergyCertificate.wrapAsEnergyCertificateRelatedProperties()
      .setBuildingEnergyRatingType((BuildingEnergyRatingType) value);
    }

    @Override
    public Object get(RealEstateWithEnergyCertificate realEstateWithEnergyCertificate) {
      return realEstateWithEnergyCertificate.wrapAsEnergyCertificateRelatedProperties().getBuildingEnergyRatingType();
    }
  },

  //baujahr
  EV_CONSTRUCTION_DATE(new NumberValidationRule(EnergyCertificateValidationErrorCodes.CONSTRUCTION_YEAR_INVALID,
      EnergyCertificateValidationConstants.YEAR_MIN_VALUE, EnergyCertificateValidationConstants.YEAR_MAX_VALUE),
    "constructionYear", EnergyCertificateValidationErrorCodes.EV_CONSTRUCTION_DATE_NOT_VALID_FOR_THIS_CERTIFICATE) {
    @Override
    public void set(RealEstateWithEnergyCertificate realEstateWithEnergyCertificate, Object value) {
      realEstateWithEnergyCertificate.wrapAsEnergyCertificateRelatedProperties().setConstructionYear((Integer) value);
    }

    @Override
    public Object get(RealEstateWithEnergyCertificate realEstateWithEnergyCertificate) {
      return realEstateWithEnergyCertificate.wrapAsEnergyCertificateRelatedProperties().getConstructionYear();
    }
  },

  //nicht bekannt
  EV_CONSTRUCTION_DATE_UNKNOWN(new EmptyValidationRule(), "constructionYearUnknown",
    EnergyCertificateValidationErrorCodes.EV_CONSTRUCTION_DATE_UNKNOWN_NOT_VALID_FOR_THIS_CERTIFICATE) {
    @Override
    public void set(RealEstateWithEnergyCertificate realEstateWithEnergyCertificate, Object value) {
      if (value == null) {
        value = Boolean.FALSE;
      }
      realEstateWithEnergyCertificate.wrapAsEnergyCertificateRelatedProperties()
      .setConstructionYearUnknown((Boolean) value);
    }

    @Override
    public Object get(RealEstateWithEnergyCertificate realEstateWithEnergyCertificate) {
      return realEstateWithEnergyCertificate.wrapAsEnergyCertificateRelatedProperties().getConstructionYearUnknown();
    }
  },

  //befeuerungsart
  EV_FIRING_TYPE(new EmptyValidationRule(), "firingTypes", EV_FIRING_TYPE_NOT_VALID_FOR_THIS_CERTIFICATE) {
    @Override
    public void set(RealEstateWithEnergyCertificate realEstateWithEnergyCertificate, Object value) {
      realEstateWithEnergyCertificate.wrapAsEnergyCertificateRelatedProperties()
      .setFiringTypes((Set<FiringType>) value);
    }

    @Override
    public Object get(RealEstateWithEnergyCertificate realEstateWithEnergyCertificate) {
      return realEstateWithEnergyCertificate.wrapAsEnergyCertificateRelatedProperties().getFiringTypes();
    }
  },

  //heizungstype
  EV_HEATING_TYPE(new EmptyValidationRule(), "heatingType",
    EnergyCertificateValidationErrorCodes.EV_HEATING_TYPE_NOT_VALID_FOR_THIS_CERTIFICATE) {
    @Override
    public void set(RealEstateWithEnergyCertificate realEstateWithEnergyCertificate, Object value) {
      realEstateWithEnergyCertificate.wrapAsEnergyCertificateRelatedProperties().setHeatingType((HeatingType) value);
    }

    @Override
    public Object get(RealEstateWithEnergyCertificate realEstateWithEnergyCertificate) {
      return realEstateWithEnergyCertificate.wrapAsEnergyCertificateRelatedProperties().getHeatingType();
    }
  },

  //Energieeffizienzklasse
  EV_ENERGY_EFFICENCY_CLASS(new EmptyValidationRule(), "energyCertificateInformation", "energyEfficiencyClass",
    EnergyCertificateValidationErrorCodes.EV_EFFICIENCY_CLASS_NOT_VALID_FOR_THIS_CERTIFICATE) {
    @Override
    public void set(RealEstateWithEnergyCertificate realEstateWithEnergyCertificate, Object value) {
      realEstateWithEnergyCertificate.wrapAsEnergyCertificateRelatedProperties()
      .setEnergyEfficiencyClass((EnergyEfficiencyClass) value);
    }

    @Override
    public Object get(RealEstateWithEnergyCertificate realEstateWithEnergyCertificate) {
      return realEstateWithEnergyCertificate.wrapAsEnergyCertificateRelatedProperties().getEnergyEfficiencyClass();
    }
  },

  //Stromverbrauchskennwert
  EV_ELECTRICITY_CONSUMPTION(new NumberValidationRule(
      NUMBER_IS_OUT_OF_RANGE,
      NUMBER_MIN_VALUE,
      NUMBER_MAX_VALUE), "energyCertificateInformation", "electricityConsumption",
    EnergyCertificateValidationErrorCodes.EV_ELECTRICITY_CONSUMPTION_NOT_VALID_FOR_THIS_CERTIFICATE) {
    @Override
    public void set(RealEstateWithEnergyCertificate realEstateWithEnergyCertificate, Object value) {
      realEstateWithEnergyCertificate.wrapAsEnergyCertificateRelatedProperties()
      .setElectricityConsumption((BigDecimal) value);
    }

    @Override
    public Object get(RealEstateWithEnergyCertificate realEstateWithEnergyCertificate) {
      return realEstateWithEnergyCertificate.wrapAsEnergyCertificateRelatedProperties().getElectricityConsumption();
    }
  },

  //Heizenergieverbrauchskennwert
  EV_HEATING_CONSUMPTION(new NumberValidationRule(
      NUMBER_IS_OUT_OF_RANGE,
      NUMBER_MIN_VALUE,
      NUMBER_MAX_VALUE), "energyCertificateInformation", "heatingConsumption",
    EnergyCertificateValidationErrorCodes.EV_HEATING_CONSUMPTION_NOT_VALID_FOR_THIS_CERTIFICATE) {
    @Override
    public void set(RealEstateWithEnergyCertificate realEstateWithEnergyCertificate, Object value) {
      realEstateWithEnergyCertificate.wrapAsEnergyCertificateRelatedProperties()
      .setHeatingConsumption((BigDecimal) value);
    }

    @Override
    public Object get(RealEstateWithEnergyCertificate realEstateWithEnergyCertificate) {
      return realEstateWithEnergyCertificate.wrapAsEnergyCertificateRelatedProperties().getHeatingConsumption();
    }
  },

  //Endenergieverbrauch Strom
  EV_ENERGY_CONSUMPTION_ELECTRICITY(new NumberValidationRule(
      NUMBER_IS_OUT_OF_RANGE,
      NUMBER_MIN_VALUE, NUMBER_MAX_VALUE), "energyCertificateInformation", "energyConsumptionElectricity",
    EnergyCertificateValidationErrorCodes.EV_ENERGY_CONSUMPTION_ELECTRICITY_NOT_VALID_FOR_THIS_CERTIFICATE) {
    @Override
    public void set(RealEstateWithEnergyCertificate realEstateWithEnergyCertificate, Object value) {
      realEstateWithEnergyCertificate.wrapAsEnergyCertificateRelatedProperties()
      .setEnergyConsumptionElectricity((BigDecimal) value);
    }

    @Override
    public Object get(RealEstateWithEnergyCertificate realEstateWithEnergyCertificate) {
      return realEstateWithEnergyCertificate.wrapAsEnergyCertificateRelatedProperties()
        .getEnergyConsumptionElectricity();
    }
  },

  //Endenergieverbrauch Warme
  EV_ENERGY_CONSUMPTION_HEATING(new NumberValidationRule(
      NUMBER_IS_OUT_OF_RANGE,
      NUMBER_MIN_VALUE,
      NUMBER_MAX_VALUE), "energyCertificateInformation", "energyConsumptionHeating",
    EnergyCertificateValidationErrorCodes.EV_ENERGY_CONSUMPTION_HEATING_NOT_VALID_FOR_THIS_CERTIFICATE) {
    @Override
    public void set(RealEstateWithEnergyCertificate realEstateWithEnergyCertificate, Object value) {
      realEstateWithEnergyCertificate.wrapAsEnergyCertificateRelatedProperties()
      .setEnergyConsumptionHeating((BigDecimal) value);
    }

    @Override
    public Object get(RealEstateWithEnergyCertificate realEstateWithEnergyCertificate) {
      return realEstateWithEnergyCertificate.wrapAsEnergyCertificateRelatedProperties().getEnergyConsumptionHeating();
    }
  },

  //Endenergiebedarf Strom
  EV_THERMAL_CHARACTERISTIC_ELECTRICITY(new NumberValidationRule(NUMBER_IS_OUT_OF_RANGE,
      NUMBER_MIN_VALUE, NUMBER_MAX_VALUE), "energyCertificateInformation", "thermalCharacteristicElectricity",
    EnergyCertificateValidationErrorCodes.EV_THERMAL_CHARACTERISTIC_ELECTRICITY_NOT_VALID_FOR_THIS_CERTIFICATE) {
    @Override
    public void set(RealEstateWithEnergyCertificate realEstateWithEnergyCertificate, Object value) {
      realEstateWithEnergyCertificate.wrapAsEnergyCertificateRelatedProperties()
      .setThermalCharacteristicElectricity((BigDecimal) value);
    }

    @Override
    public Object get(RealEstateWithEnergyCertificate realEstateWithEnergyCertificate) {
      return realEstateWithEnergyCertificate.wrapAsEnergyCertificateRelatedProperties()
        .getThermalCharacteristicElectricity();
    }
  },

  //Endenergiebedarf Warme
  EV_THERMAL_CHARACTERISTIC_HEATING(new NumberValidationRule(NUMBER_IS_OUT_OF_RANGE,
      NUMBER_MIN_VALUE, NUMBER_MAX_VALUE), "energyCertificateInformation", "thermalCharacteristicHeating",
    EnergyCertificateValidationErrorCodes.EV_THERMAL_CHARACTERISTIC_HEATING_NOT_VALID_FOR_THIS_CERTIFICATE) {
    @Override
    public void set(RealEstateWithEnergyCertificate realEstateWithEnergyCertificate, Object value) {
      realEstateWithEnergyCertificate.wrapAsEnergyCertificateRelatedProperties()
      .setThermalCharacteristicHeating((BigDecimal) value);
    }

    @Override
    public Object get(RealEstateWithEnergyCertificate realEstateWithEnergyCertificate) {
      return realEstateWithEnergyCertificate.wrapAsEnergyCertificateRelatedProperties()
        .getThermalCharacteristicHeating();
    }
  };

  private final List<EnergyCertificateValidationRule> energyCertificateValidationRules = new ArrayList<>();
  private final String energyCertificateInformationField;
  private final String energyCertificateInformationNestedClass;
  private final String bussinesFieldValidationCode;

  EnergyCertificateField(EnergyCertificateValidationRule energyCertificateValidationRule,
                         String energyCertificateInformationField,
                         String bussinesFieldValidationCode) {
    this.bussinesFieldValidationCode = notBlank("bussinesFieldValidationCode", bussinesFieldValidationCode);
    this.energyCertificateValidationRules.add(energyCertificateValidationRule);
    this.energyCertificateInformationNestedClass = null;
    this.energyCertificateInformationField = notBlank("energyCertificateInformationField",
      energyCertificateInformationField);
  }

  EnergyCertificateField(EnergyCertificateValidationRule energyCertificateValidationRule,
                         String energyCertificateInformationNestedClass,
                         String energyCertificateInformationField, String bussinesFieldValidationCode) {
    energyCertificateValidationRules.add(energyCertificateValidationRule);
    this.energyCertificateInformationNestedClass = energyCertificateInformationNestedClass;
    this.energyCertificateInformationField = notBlank("energyCertificateInformationField",
      energyCertificateInformationField);
    this.bussinesFieldValidationCode = notBlank("bussinesFieldValidationCode", bussinesFieldValidationCode);

  }

  public List<EnergyCertificateValidationRule> getEnergyCertificateValidationRules() {
    return Collections.unmodifiableList(energyCertificateValidationRules);
  }

  public String getEnergyCertificateInformationField() {
    return energyCertificateInformationField;
  }

  public static EnergyCertificateField findFieldByName(String fieldName) {
    for (EnergyCertificateField energyCertificateField : EnergyCertificateField.values()) {
      final String informationField = energyCertificateField.getEnergyCertificateInformationField();
      if ((informationField != null) && (informationField).equals(fieldName)) {
        return energyCertificateField;
      }
    }
    return null;
  }

  public abstract void set(RealEstateWithEnergyCertificate realEstateWithEnergyCertificate, Object value);

  public abstract Object get(RealEstateWithEnergyCertificate realEstateWithEnergyCertificate);

  public String getFieldName() {
    return energyCertificateInformationField;
  }

  public String getEnergyCertificateInformationNestedClass() {
    return energyCertificateInformationNestedClass;
  }

  public String getBussinesFieldValidationCode() {
    return bussinesFieldValidationCode;
  }

  @Override
  public String toString() {
    return "EnergyCertificateField{" +
      "energyCertificateValidationRules=" + energyCertificateValidationRules +
      ", energyCertificateInformationField='" + energyCertificateInformationField + '\'' +
      ", energyCertificateInformationNestedClass='" + energyCertificateInformationNestedClass + '\'' +
      ", bussinesFieldValidationCode='" + bussinesFieldValidationCode + '\'' +
      '}';
  }
}
