package de.is24.realEstates.domain.energycertificate.validation;

import de.is24.realEstates.domain.energyCertificate.EnergyCertificateCreationDate;
import de.is24.realEstates.domain.living.BuildingEnergyRatingType;
import java.util.Collections;
import java.util.EnumSet;
import java.util.HashSet;
import java.util.Set;


public class EnergyCertificateFieldCombination {
  private final Set<EnergyCertificateField> basicFieldSet = EnumSet.of(
    EnergyCertificateField.EV_ENERGY_CERTIFICATE_CREATION_DATE,
    EnergyCertificateField.EV_ENERGY_CERTIFICATE_TYPE, EnergyCertificateField.EV_CONSTRUCTION_DATE,
    EnergyCertificateField.EV_FIRING_TYPE,
    EnergyCertificateField.EV_HEATING_TYPE);
  private final Set<EnergyCertificateField> energyDemandAfter01052014;
  private final Set<EnergyCertificateField> energyConsumptionAfter01052014;
  private final Set<EnergyCertificateField> energyDemandBefore01052014;
  private final Set<EnergyCertificateField> energyConsumptionBefore01052014;

  public EnergyCertificateFieldCombination(Set<EnergyCertificateField> energyDemandAfter01052014,
                                           Set<EnergyCertificateField> energyConsumptionAfter01052014,
                                           Set<EnergyCertificateField> energyDemandBefore01052014,
                                           Set<EnergyCertificateField> energyConsumptionBefore01052014) {
    this.energyDemandAfter01052014 = Collections.unmodifiableSet(new HashSet<EnergyCertificateField>(
      energyDemandAfter01052014));
    this.energyConsumptionAfter01052014 = Collections.unmodifiableSet(new HashSet<EnergyCertificateField>(
      energyConsumptionAfter01052014));
    this.energyDemandBefore01052014 = Collections.unmodifiableSet(new HashSet<EnergyCertificateField>(
      energyDemandBefore01052014));
    this.energyConsumptionBefore01052014 = Collections.unmodifiableSet(new HashSet<EnergyCertificateField>(
      energyConsumptionBefore01052014));
  }

  public Set<EnergyCertificateField> getValidSetForEVTypeAndDate(
    BuildingEnergyRatingType buildingEnergyRatingType, EnergyCertificateCreationDate energyCertificateCreationDate) {
    switch (buildingEnergyRatingType) {
      case NO_INFORMATION: {
        return getBasicFieldSet();
      }

      case ENERGY_CONSUMPTION: {
        return getValidSetForConsumptionDate(energyCertificateCreationDate);
      }

      case ENERGY_REQUIRED: {
        return getValidSetForDemandDate(energyCertificateCreationDate);
      }

      default: {
        return getBasicFieldSet();
      }
    }
  }

  private Set<EnergyCertificateField> getValidSetForDemandDate(
    EnergyCertificateCreationDate energyCertificateCreationDate) {
    switch (energyCertificateCreationDate) {
      case NO_INFORMATION: {
        return getBasicFieldSet();
      }

      case FROM_01_05_2014: {
        return getEnergyCertificateFields(energyDemandAfter01052014);
      }

      case TILL_30_04_2014: {
        return getEnergyCertificateFields(energyDemandBefore01052014);
      }

      default: {
        return getBasicFieldSet();
      }
    }
  }

  private Set<EnergyCertificateField> getEnergyCertificateFields(Set<EnergyCertificateField> setToBeAdded) {
    Set<EnergyCertificateField> tmpSet = getBasicFieldSet();
    tmpSet.addAll(setToBeAdded);
    return Collections.unmodifiableSet(tmpSet);
  }

  private Set<EnergyCertificateField> getValidSetForConsumptionDate(
    EnergyCertificateCreationDate energyCertificateCreationDate) {
    switch (energyCertificateCreationDate) {
      case NO_INFORMATION: {
        return getBasicFieldSet();
      }

      case FROM_01_05_2014: {
        return getEnergyCertificateFields(energyConsumptionAfter01052014);
      }

      case TILL_30_04_2014: {
        return getEnergyCertificateFields(energyConsumptionBefore01052014);
      }

      default: {
        return getBasicFieldSet();
      }
    }
  }

  public Set<EnergyCertificateField> getBasicFieldSet() {
    return new HashSet<>(basicFieldSet);
  }
}
